package ch.hslu.appe.usermanagement.model;

/**
 * Place.
 */
public final class Place {

    private int id;
    private String canton;
    private Double latitude;
    private Double longitude;
    private String name;
    private String shortName;
    private int zipCodeType;
    private int zipCode;
    private int zipCodeExtension;

    /**
     * Calculate the distance to the destination.
     * @param destination destination place
     * @return distance
     */
    public float distanceTo(final Place destination) {

        if (latitude == null || longitude == null) {
            return Float.POSITIVE_INFINITY;
        }

        // Sample from:
        // http://stackoverflow.com/questions/837872/
        // calculate-distance-in-meters-when-you-know-longitude-and-latitude-in-java
        final double earthRadius = 3958.75;
        final double distanceLatitude = Math.toRadians(destination.getLatitude() - latitude);
        final double distanceLongitude = Math.toRadians(destination.getLongitude() - longitude);
        final double a = Math.sin(distanceLatitude / 2) * Math.sin(distanceLatitude / 2)
                + Math.cos(Math.toRadians(latitude)) * Math.cos(Math.toRadians(destination.getLatitude()))
                * Math.sin(distanceLongitude / 2) * Math.sin(distanceLongitude / 2);
        final double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        final double dist = earthRadius * c;

        final int meterConversion = 1609;

        return (float) (dist * meterConversion);
    }

    @Override
    public boolean equals(final Object other) {
        // Check for same instance
        if (this == other) {
            return true;
        }

        // Check for null values
        if (other == null) {
            return false;
        }

        // Check for same type
        if (other.getClass() != getClass()) {
            return false;
        }

        // Check properties
        return ((Place) other).getId() == getId() && ((Place) other).getZipCode() == getZipCode();
    }

    /**
     * Return the Canton.
     * @return canton
     */
    public String getCanton() {
        return canton;
    }

    /**
     * Return the ID of this place.
     * @return id
     */
    public int getId() {
        return id;
    }

    /**
     * Return the latitude of this place.
     * @return latitude
     */
    public double getLatitude() {
        if (latitude == null) {
            return 0;
        }

        return latitude.doubleValue();
    }

    /**
     * Return Longitude of this place.
     * @return longitude
     */
    public double getLongitude() {
        if (longitude == null) {
            return 0;
        }

        return longitude.doubleValue();
    }

    /**
     * Return the Name.
     * @return name.
     */
    public String getName() {
        return name;
    }

    /**
     * Return the Shortname.
     * @return shortname
     */
    public String getShortName() {
        return shortName;
    }

    /**
     * Return Zip Code.
     * @return zip code
     */
    public int getZipCode() {
        return zipCode;
    }

    /**
     * Return ZipCode Extension.
     * @return zip code extension
     */
    public int getZipCodeExtension() {
        return zipCodeExtension;
    }

    /**
     * Return ZipCode Type.
     * @return zip code type
     */
    public int getZipCodeType() {
        return zipCodeType;
    }

    @Override
    public int hashCode() {
        return zipCode;
    }

    /**
     * Sets the Canton.
     * @param canton to be set
     */
    public void setCanton(final String canton) {
        this.canton = canton;
    }

    /**
     * Sets the ID.
     * @param id to be set
     */
    public void setId(final int id) {
        this.id = id;
    }

    /**
     * Sets the Latitude.
     * @param latitude to be set
     */
    public void setLatitude(final Double latitude) {
        this.latitude = latitude;
    }

    /**
     * Sets Longitude.
     * @param longitude to be set
     */
    public void setLongitude(final Double longitude) {
        this.longitude = longitude;
    }

    /**
     * Set Name.
     * @param name to be set
     */
    public void setName(final String name) {
        this.name = name;
    }

    /**
     * Set ShortName.
     * @param shortName to be set
     */
    public void setShortName(final String shortName) {
        this.shortName = shortName;
    }

    /**
     * Set ZipCode.
     * @param zipCode to be set
     */
    public void setZipCode(final int zipCode) {
        this.zipCode = zipCode;
    }

    /**
     * Set ZipCodeExtension.
     * @param zipCodeExtension to be set
     */
    public void setZipCodeExtension(final int zipCodeExtension) {
        this.zipCodeExtension = zipCodeExtension;
    }

    /**
     * Set ZipCode Type.
     * @param type to be set
     */
    public void setZipCodeType(final int type) {
        this.zipCodeType = type;
    }

    @Override
    public String toString() {
        return zipCode + " " + name;
    }
}
