package ch.hslu.appe.usermanagement.persistency;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Session;

import ch.hslu.appe.persistency.Persistency;
import ch.hslu.appe.usermanagement.exception.CacheOutdatedException;
import ch.hslu.appe.usermanagement.filter.PlaceFilter;
import ch.hslu.appe.usermanagement.model.Place;

/**
 * Place Persistency Implementation.
 */
public final class PlacePersistencyImpl extends Persistency implements PlacePersistency {

    private static ArrayList<Place> placesCache;

    /**
     * Constructor.
     */
    public PlacePersistencyImpl() {

        super("UserManagementData");
    }

    @Override
    public Place getPlaceById(final int id) throws CacheOutdatedException {
        // Update the cache if necessary
        updateCache();

        // Lookup for the place with the same id
        for (Place place : placesCache) {
            if (place.getId() == id) {

                // Add log trace entry
                logger.trace("Place '" + place + "' found by it's id.");

                return place;
            }
        }

        return null;
    }

    @Override
    public Place getPlaceByZipCode(final int zipCode) throws CacheOutdatedException {
        // Update the cache if necessary
        updateCache();

        // Lookup for the place with the same zip code
        for (Place place : placesCache) {
            if (place.getZipCode() == zipCode) {

                // Add log trace entry
                logger.trace("Place '" + place + "' found by it's zip code.");

                return place;
            }
        }

        return null;
    }

    @Override
    public ArrayList<Place> getPlaces() {

        if (placesCache == null) {
            initializeCache();
        }

        return placesCache;
    }

    @Override
    public ArrayList<Place> getPlaces(final PlaceFilter filter) {

        ArrayList<Place> places = new ArrayList<Place>();

        // Add users to the typed generic list
        for (Object place : placesCache) {

            if (filter == null || filter.match((Place) place)) {
                places.add((Place) place);
            }
        }

        // Add log trace entry
        logger.trace("Fetched " + places.size() + " places sucessfully");

        return places;
    }

    @Override
    public ArrayList<Place> getPlacesWithinRadius(final Place origin, final int meters) throws CacheOutdatedException {
        // Update the cache if necessary
        updateCache();

        final ArrayList<Place> placesWithinRadius = new ArrayList<Place>();

        // Lookup for places within
        for (Place place : placesCache) {
            if (!place.equals(origin) && place.distanceTo(origin) < meters) {
                placesWithinRadius.add(place);
            }
        }

        // Add log trace entry
        logger.trace(placesWithinRadius.size() + " places found within a radius of " + meters + "m of the place "
                + origin + ".");

        return placesWithinRadius;
    }

    @Override
    public boolean placeExists(final int zipCode) throws CacheOutdatedException {

        // Update the cache if necessary
        updateCache();

        // Lookup for places with the same zip code
        for (Place place : placesCache) {
            if (place.getZipCode() == zipCode) {

                // Add log trace entry
                logger.trace("Place " + place + " exists");

                return true;
            }
        }

        // Add log trace entry
        logger.trace("Place with zip-code " + zipCode + " does not exists");

        return false;
    }

    @Override
    public boolean placeExists(final Place place) throws CacheOutdatedException {
        // Update the cache if necessary
        updateCache();

        // Lookup for equal places within the cache
        for (Place item : placesCache) {
            if (item.equals(place)) {

                // Add log trace entry
                logger.trace("Place " + place + " exists");

                return true;
            }
        }

        // Add log trace entry
        logger.trace("Place " + place + " does not exists");

        return false;
    }

    /**
     * Initializes the places cache.
     */
    private void initializeCache() {
        final Session session = sessionFactory.openSession();
        final ArrayList<Place> places = new ArrayList<Place>();

        try {

            // Fetch all db places
            @SuppressWarnings("rawtypes")
            final List fetchedPlaces = session.createCriteria(Place.class).list();

            // Add places to the typed generic list
            for (Object place : fetchedPlaces) {
                places.add((Place) place);
            }

            // Add log trace entry
            logger.trace("Fetched " + places.size() + " places sucessfully.");
        } finally {
            session.close();
        }

        logger.trace("Places-cache initialized successfully.");

        // save cache
        placesCache = places;
    }

    /**
     * Updates the cache if necessary.
     * @throws CacheOutdatedException Throws the exception, if wasn't possible to update the cache
     */
    private void updateCache() throws CacheOutdatedException {
        if (placesCache == null) {
            initializeCache();

            if (placesCache == null) {
                throw new CacheOutdatedException();
            }
        }
    }
}
