package ch.hslu.appe.usermanagement.persistency;

import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import ch.hslu.appe.persistency.Persistency;
import ch.hslu.appe.usermanagement.filter.UserFilter;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.User;

/**
 * User Persistency Implementation.
 */
public class UserPersistencyImpl extends Persistency implements UserPersistency {

    /**
     * Constructor.
     */
    public UserPersistencyImpl() {

        super("UserManagementData");
    }

    /**
     * Persists a new user.
     * @param user The user object
     * @return Returns true on success, otherwise false
     */
    @Override
    public final boolean addUser(final User user) {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            user.setId((Integer) session.save(user));

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Add log trace entry
            logger.trace("Persisted a new user '" + user + "' successfully");

            // Flush, to make sure, that the object is saved
            session.flush();
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to save the user '" + user + "'.", e);

            return false;
        } finally {

            session.close();
        }

        return true;
    }

    @Override
    public final boolean deleteUser(final int id) {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        if (!deleteUserAddresses(id)) {
            return false;
        }

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            // Create a hql query and execute it against the db
            final String hql = "delete from " + User.class.getName() + " where id= :id";
            session.createQuery(hql).setInteger("id", id).executeUpdate();

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Deleted the user with the id '" + id + "' sucessfully");
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to update the user with the id '" + id + "'.", e);

            return false;
        } finally {

            session.close();
        }

        return true;
    }

    @Override
    public final boolean deleteUser(final String login) {

        // Because we need the users id to delete his addresses, we must get the
        // user from the db first
        final User user = getUser(login);

        if (user == null) {
            return false;
        }

        return deleteUser(user.getId());
    }

    @Override
    public final boolean deleteUserAddresses(final int userId) {
        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            // Create a hql query and execute it against the db
            final String hql = "delete from " + Address.class.getName() + " where userid= :userid";
            session.createQuery(hql).setInteger("userid", userId).executeUpdate();

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Deleted all addresses of the user with the id '" + userId + "' sucessfully");
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to delete the addresses of the user with the id '" + userId + "'.", e);

            return false;
        } finally {
            session.close();
        }

        return true;
    }

    /**
     * Gets a user from the database by its unique id.
     * @param id the user ID
     * @return Returns an user object if the user exists, otherwise null
     */
    @Override
    public final User getUser(final int id) {

        // Check validity of the user id
        if (id <= 0) {
            return null;
        }

        final Session session = sessionFactory.openSession();

        try {
            // Get the user by its primary key
            final User user = (User) session.get(User.class, id);

            // Add log trace entry
            logger.trace("Fetched the user '" + user + "' sucessfully");

            return user;
        } finally {

            session.close();
        }
    }

    /**
     * Gets a user from the database by its unique login name.
     * @param login The login name
     * @return Returns an user object if the user exists, otherwise null
     */
    @Override
    public final User getUser(final String login) {

        // Ignore empty login names
        if (login == null || login.isEmpty()) {
            return null;
        }

        final Session session = sessionFactory.openSession();

        try {
            final Criteria criteria = session.createCriteria(User.class);

            // Add the criteria (login name must match)
            criteria.add(Restrictions.eq("login", login));

            final User user = (User) criteria.uniqueResult();

            // Add log trace entry
            logger.trace("Fetched the user '" + user + "' sucessfully");

            return user;
        } finally {

            session.close();
        }
    }

    @Override
    public final ArrayList<User> getUsers() {

        return getUsers(null);
    }

    @SuppressWarnings("rawtypes")
    @Override
    public final ArrayList<User> getUsers(final UserFilter filter) {

        final Session session = sessionFactory.openSession();
        final ArrayList<User> users = new ArrayList<User>();

        try {
            final Criteria criteria = session.createCriteria(User.class);

            // Add the filter
            if (filter != null) {
                for (Entry<String, Object> item : filter.getFilter().entrySet()) {
                    if (item.getValue() instanceof String) {
                        criteria.add(Restrictions.like(item.getKey(), item.getValue()));
                    } else {
                        criteria.add(Restrictions.eq(item.getKey(), item.getValue()));
                    }
                }
            }

            // Fetch all db users
            final List fetchedUsers = criteria.list();

            // Add users to the typed generic list
            for (Object user : fetchedUsers) {
                users.add((User) user);
            }

            // Add log trace entry
            logger.trace("Fetched " + users.size() + " users sucessfully");
        } finally {

            session.close();
        }

        return users;
    }

    /**
     * Updates a user in the database.
     * @param user The user object
     * @return Returns true if the user was successfully updated, otherwise false
     */
    @Override
    public final boolean updateUser(final User user) {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            session.update(user);

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Updated the user '" + user + "' sucessfully");
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to update the user '" + user + "'.", e);

            return false;
        } finally {
            session.close();
        }

        return true;
    }

    /**
     * Returns a value that indicates whether the specified user exists in the database.
     * @param userId The user ID
     * @return Returns true if the user exists in the database, otherwise false
     */
    @Override
    public final boolean userExists(final int userId) {

        // Check for invalid values
        if (userId <= 0) {
            return false;
        }

        final Session session = sessionFactory.openSession();

        try {
            final Query query = session.createQuery("select 1 from User t where t.id = :id");

            query.setInteger("id", userId);

            // If there's at least one result, return true, otherwise false
            final boolean exists = (query.uniqueResult() != null);

            // Add log trace entry
            logger.trace("Check for user with the id '" + userId + "' existency: " + exists);

            return exists;
        } finally {

            session.close();
        }
    }

    /**
     * Returns a value that indicates whether the specified user exists in the database.
     * @param user The user object
     * @return Returns true if the user exists in the database, otherwise false
     */
    @Override
    public final boolean userExists(final User user) {

        // Check for null values
        if (user == null) {
            return false;
        }

        // Check validity of the user id and login
        if (user.getId() <= 0 && user.getLogin().isEmpty()) {
            return false;
        }

        final Session session = sessionFactory.openSession();

        try {
            final Query query = session.createQuery("select 1 from User t where t.id = :id OR t.login = :login");

            query.setInteger("id", user.getId());
            query.setString("login", user.getLogin());

            // If there's at least one result, return true, otherwise false
            final boolean exists = (query.uniqueResult() != null);

            // Add log trace entry
            logger.trace("Check for user '" + user + "' existency: " + exists);

            return exists;
        } finally {

            session.close();
        }
    }
}
