package ch.hslu.appe.usermanagement.persistency;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.model.UserState;
import ch.hslu.appe.usermanagement.persistency.UserPersistencyImpl;

/**
 * User Persistency Implementation Test.
 */
public class UserPersistencyImplTest {

    private UserPersistencyImpl persistency;

    // consider using a Map<String,Boolean> to say whether the identifier is
    // being used or not
    private final Set<String> identifiers = new HashSet<String>();

    private final String lexicon = "ABCDEFGHIJKLMNOPQRSTUVWXYZ12345674890".toLowerCase();
    private final java.util.Random rand = new java.util.Random();

    /**
     * Create a Random LoginName.
     * @return new Username
     */
    private String createRandomLogin() {

        StringBuilder builder = new StringBuilder();

        // Creates a random unique string
        while (builder.length() == 0) {

            int length = rand.nextInt(5) + 5;
            for (int i = 0; i < length; i++) {
                builder.append(lexicon.charAt(rand.nextInt(lexicon.length())));
            }

            if (identifiers.contains(builder.toString())) {
                builder = new StringBuilder();
            }
        }

        identifiers.add(builder.toString());

        return builder.toString();
    }

    /**
     * Create RandomUser.
     * @return generated random User
     * @throws IllegalNameException Illegal Username
     */
    private User createRandomUser() throws IllegalNameException {
        final User user = new User();
        user.setLogin(createRandomLogin());
        user.setSurname("TestData");
        user.setPrename("TestData");
        user.setSalutation("TestData");
        user.setState(UserState.ACTIVE);
        user.setPassword("TestData");
        user.setRole("Abwascher");

        return user;
    }

    /**
     * JUnit Setup.
     */
    @Before
    public final void setUp() {

        persistency = new UserPersistencyImpl();
    }

    /**
     * JUnit TearDown.
     */
    @After
    public final void tearDown() {

        // Delete all the inserted users
        final ArrayList<User> users = persistency.getUsers();

        for (User user : users) {
            // Consider only users generated by this test
            if (!identifiers.contains(user.getLogin())) {
                continue;
            }

            persistency.deleteUser(user.getId());
        }
    }

    /**
     * Tests to add a new User.
     * @throws IllegalNameException Illegal Username
     */
    @Test
    public final void testAddUser() throws IllegalNameException {

        final User user = createRandomUser();

        // Insert the user
        persistency.addUser(user);

        // User must have a generated id after being inserted
        Assert.assertNotEquals(user.getId(), 0);
    }

    /**
     * Tests to add a duplicat Username.
     * @throws IllegalNameException Illegal Username
     */
    @Test
    public final void testAddUserDuplicate() throws IllegalNameException {

        final User userOne = createRandomUser();
        final User userTwo = createRandomUser();

        // Set the same user login
        userTwo.setLogin(userOne.getLogin());

        // Insert the generated users
        Assert.assertTrue(persistency.addUser(userOne));
        Assert.assertFalse(persistency.addUser(userTwo));
    }

    /**
     * Tests get a User by ID.
     * @throws IllegalNameException illegal Username
     */
    @Test
    public final void testGetUserById() throws IllegalNameException {

        final User user = createRandomUser();

        // Insert the user
        persistency.addUser(user);

        final User dbUser = persistency.getUser(user.getId());

        // User must exists
        Assert.assertEquals(dbUser, user);

        // Check other properties
        Assert.assertEquals(dbUser.getPassword(), user.getPassword());
        Assert.assertEquals(dbUser.getPrename(), user.getPrename());
        Assert.assertEquals(dbUser.getSurname(), user.getSurname());
        Assert.assertEquals(dbUser.getRole(), user.getRole());
        Assert.assertEquals(dbUser.getState(), user.getState());
        Assert.assertEquals(dbUser.getSalutation(), user.getSalutation());
    }

    /**
     * Test get User by empty Login.
     */
    @Test
    public final void testGetUserEmptyLogin() {
        Assert.assertNull(persistency.getUser(""));
    }

    /**
     * Test get user by Invalid ID.
     */
    @Test
    public final void testGetUserInvalidId() {
        Assert.assertNull(persistency.getUser(-1));
    }

    /**
     * Test User Exists by User.
     * @throws IllegalNameException illegal Username
     */
    @Test
    public final void testUserExists() throws IllegalNameException {

        final User user = createRandomUser();

        // Insert the user
        persistency.addUser(user);

        // User must exists
        Assert.assertTrue(persistency.userExists(user));
    }

    /**
     * Test User Exists by ID.
     * @throws IllegalNameException illegal Username
     */
    @Test
    public final void testUserExistsById() throws IllegalNameException {

        final User user = createRandomUser();

        // Insert the user
        persistency.addUser(user);

        final User lookupUser = new User();
        lookupUser.setId(user.getId());

        // User must exists
        Assert.assertTrue(persistency.userExists(lookupUser));
    }

    /**
     * Test User Exists by Login.
     * @throws IllegalNameException illegal Username
     */
    @Test
    public final void testUserExistsByLogin() throws IllegalNameException {

        final User user = createRandomUser();

        // Insert the user
        persistency.addUser(user);

        final User lookupUser = new User();
        lookupUser.setLogin(user.getLogin());

        // User must exists
        Assert.assertTrue(persistency.userExists(lookupUser));
    }
}
